/// <reference types="mongoose" />
import { Document, Model as MModel, Mongoose, Schema } from 'mongoose';
import { MongoError } from 'mongodb';
import { ModelConfig } from './model-config';
export interface Model extends Document {
}
export declare type ModelType = typeof Model;
export declare type IModel<E extends Model> = MModel<E>;
/**
 * Wrapped Model from mongoose.Model.
 */
export declare class Model {
    static $CONFIG: ModelConfig;
    static $SCHEMA: {};
    private static $PRES;
    private static $POSTS;
    private static $VIRTUALS;
    private static $PLUGINS;
    private static $INDEXES;
    private static $MIXINS;
    static _mongooseOptions: MongooseOptions;
    static Config(config: ModelConfig): ModelType;
    static Schema(schema: {}): ModelType;
    static Pre(pre: Pre): ModelType;
    static Post(post: Post): ModelType;
    static Virtual(virtual: Virtual): ModelType;
    static Plugin(plugin: Plugin): ModelType;
    static Index(index: Index): ModelType;
    static Mixin(model: ModelType): ModelType;
    static _$modifySchema(discriminatorKey: string): void;
    static $mongooseOptions(): MongooseOptions;
    static $register<D extends Document, M>(mongooseInstance?: Mongoose): MModel<D> & M;
    static _$initialize(): void;
    static cast<D extends Model>(): IModel<D>;
}
/**
 * Mongoose options for current model.
 */
export interface MongooseOptions {
    initialized: boolean;
    config?: ModelConfig;
    schema?: {};
    mongooseSchema?: Schema;
    pres?: Pre[];
    posts?: Post[];
    virtuals?: Virtual[];
    methods?: Method[];
    statics?: Method[];
    plugins?: Plugin[];
    indexes?: Index[];
}
export interface Pre {
    name: string;
    fn: (next: (err?: NativeError) => void) => void;
    parallel?: boolean;
    errorCb?: (err: Error) => void;
}
export interface Post {
    name: string;
    fn: PostFn1 | PostFn2;
}
export declare type PostFn1 = (doc: object, next: (err?: NativeError) => void) => void;
export declare type PostFn2 = (error: MongoError, doc: object, next: (err?: NativeError) => void) => void;
export interface Virtual {
    name: string;
    get?: Function;
    set?: Function;
}
export interface Method {
    name: string;
    fn: Function;
}
export interface Plugin {
    fn: (schema: Schema, options?: Object) => void;
    options?: Object;
    priority?: number;
}
export interface Index {
    fields: Object;
    options?: {
        expires?: string;
        [other: string]: any;
    };
}
export declare class NativeError extends global.Error {
}
export declare var preQueries: string[];
